QuickStrat = LibStub("AceAddon-3.0"):NewAddon("QuickStrat", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("QuickStrat")
local options = {
    name = "QuickStrat",
    handler = QuickStrat,
    type = "group",
    args = {
        msg = {
            type = "input",
            name = L["Message"],
            desc = L["The message to be displayed when you get home."],
            usage = L["<Your message>"],
            get = "GetMessage",
            set = "SetMessage",
        },
        showInChat = {
            type = "toggle",
            name = L["Show in Chat"],
            desc = L["Toggles the display of the message in the chat window."],
            get = "IsShowInChat",
            set = "ToggleShowInChat",
        },
        showOnScreen = {
            type = "toggle",
            name = L["Show on Screen"],
            desc = L["Toggles the display of the message on the screen."],
            get = "IsShowOnScreen",
            set = "ToggleShowOnScreen"
        },
    },
}

local defaults = {
    profile = {
        message = L["Welcome Home!"],
        showInChat = false,
        showOnScreen = true,
		strats = {},
    },
}

local AceGUI = LibStub("AceGUI-3.0")
local textStore

Strat = {};
Strat.__index = Strat;
function Strat:new(zone, name)
	local self = {};         -- Create a blank table
	setmetatable(self, Strat);
	self.name = name;
	self.zone = zone;
	self.next = 0;
	self.parts = {};

	if Strat.db[self.zone] and Strat.db[self.zone][self.name] then
		for i,v in ipairs(Strat.db[self.zone][self.name]["parts"]) do
			table.insert(self.parts, v);
		end
	end

	if Strat.customdb[self.zone] and Strat.customdb[self.zone][self.name] then
		for i,v in ipairs(Strat.customdb[self.zone][self.name]["parts"]) do
			table.insert(self.parts, v);
		end
	end

	return self;             -- Return the instance
end

function Strat:GetNextPart()
	if #self.parts == 0 then
		return nil;
	end
	self.next = self.next + 1;
	if self.next > #self.parts then
		self.next = 1;
	end
  	return self.parts[self.next];
end

function Strat:AddPart( part )
	if not Strat.customdb[self.zone] then
		Strat.customdb[self.zone] = {}
	end

	if not Strat.customdb[self.zone][self.name] then
		Strat.customdb[self.zone][self.name] = {
				["parts"]={}
			};
	end
	table.insert(Strat.customdb[self.zone][self.name]["parts"], part);
	table.insert(self.parts, part);
end

function QuickStrat:OnInitialize()
    -- Called when the addon is loaded
    self.db = LibStub("AceDB-3.0"):New("QuickStratDB", defaults, "Default")

    LibStub("AceConfig-3.0"):RegisterOptionsTable("QuickStrat", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuickStrat", "QuickStrat")
    self:RegisterChatCommand(L["quickstrat"], "ChatCommand")
    self:RegisterChatCommand(L["qs"], "ChatCommand")
	self.currStrat = nil;
	
	Strat.customdb = self.db.profile.strats;
	Strat.db = L["strats"];
end

QuickStrat.channelcommands = {
	s="SAY",
	say="SAY",
	p="PARTY",
	party="PARTY",
}

function QuickStrat:ChatCommand(input)
	local cmd = self:GetArgs(input, 1);

	if cmd == "config" then
		self:DrawUI();
	elseif cmd == "clear" then
		self.db.profile.strats = {};
		Strat.customdb = self.db.profile.strats;
		self:Print("Cleared all custom entries");
	elseif not cmd or cmd == "add" or QuickStrat.channelcommands[cmd] then
		if cmd == "add" then
			cmd, desc, name, zone = self:GetArgs(input, 4);
		else
			cmd, name, zone = self:GetArgs(input, 3);
		end
		
		if name then
			if not zone then
				zone = "default"
			end
		else
			name = GetUnitName("target", false);
			zone = GetZoneText();
			if name == nil then
				self:Print("Nothing targeted");
				return
			end
		end
		if self.currentStrat == nil or self.currentStrat.zone ~= zone or self.currentStrat.name ~= name then
			self.currentStrat = Strat:new(zone, name);
		end
		
		if not cmd or QuickStrat.channelcommands[cmd] then
			p = self.currentStrat:GetNextPart();
			if p == nil then
				self:Print("No strat for " .. name .. " in zone " .. zone);
			else
				self:SendChat(p, QuickStrat.channelcommands[cmd]);
			end
		elseif cmd == "add" then
			self.currentStrat:AddPart(desc);
			self:Print("Added strat to " .. name .. " in zone " .. zone);
		end
	else
		self:Print("Command not understood: " .. cmd);
    end
end

function QuickStrat:SendChat(message, channel)
	self:Print(message);
	if channel then
		SendChatMessage(message, channel);
	end
end

function QuickStrat:DrawUI()
	-- Create the frame container
	local frame = AceGUI:Create("Frame")
	frame:SetTitle("Quick Strat")
	-- frame:SetStatusText("AceGUI-3.0 Example Container Frame")
	frame:SetCallback("OnClose", function(widget, event) AceGUI:Release(widget) end)

	-- Fill Layout - the TabGroup widget will fill the whole frame
	frame:SetLayout("Fill")

	-- Create the TabGroup
	local tab =  AceGUI:Create("TabGroup")
	tab:SetLayout("Flow")
	-- Setup which tabs to show
	tab:SetTabs({{text="Tab 1", value="tab1"}, {text="Tab 2", value="tab2"}})
	-- Register callback
	tab:SetCallback("OnGroupSelected", function(container, event, group) self:SelectGroup(container, event, group) end)
	-- Set initial Tab (this will fire the OnGroupSelected callback)
	tab:SelectTab("tab1")
	
	-- add to the frame container
	frame:AddChild(tab)	
end


function QuickStrat:OnEnable()
    -- Called when the addon is enabled
end

function QuickStrat:OnDisable()
    -- Called when the addon is disabled
end

function QuickStrat:GetMessage(info)
    return self.db.profile.message
end

function QuickStrat:SetMessage(info, newValue)
    self.db.profile.message = newValue
end

function QuickStrat:IsShowInChat(info)
    return self.db.profile.showInChat
end

function QuickStrat:ToggleShowInChat(info, value)
    self.db.profile.showInChat = value
end

function QuickStrat:IsShowOnScreen(info)
    return self.db.profile.showOnScreen
end

function QuickStrat:ToggleShowOnScreen(info, value)
    self.db.profile.showOnScreen = value
end


-- function that draws the widgets for the first tab
function QuickStrat:DrawGroup1(container)
	
	tree = { 
	  { 
	    value = "A",
	    text = "Alpha",
	    icon = "Interface\\Icons\\INV_Drink_05",
	  },
	  {
	    value = "B",
	    text = "Bravo",
	    children = {
	      { 
	        value = "C", 
	        text = "Charlie"
	      },
	      {
	        value = "D",	
	        text = "Delta",
	        children = { 
	          { 
	            value = "E",
	            text = "Echo",
	          } 
	        }
	      }
	    }
	  },
	  { 
	    value = "F", 
	    text = "Foxtrot",
	    disabled = true,
	  },
	}


  local t = AceGUI:Create("TreeGroup");
  t:SetLayout("Fill")
  t:SetTree(tree);
  container:AddChild(t)
	
container:DoLayout()

  local desc = AceGUI:Create("Label")
  desc:SetText("This is Tab 2")
  desc:SetFullWidth(true)
  container:AddChild(desc)
  
end

-- function that draws the widgets for the second tab
function QuickStrat:DrawGroup2(container)
  local desc = AceGUI:Create("Label")
  desc:SetText("This is Tab 2")
  desc:SetFullWidth(true)
  container:AddChild(desc)
  
  local button = AceGUI:Create("Button")
  button:SetText("Tab 2 Button")
  button:SetWidth(200)
  container:AddChild(button)
end

-- Callback function for OnGroupSelected
function QuickStrat:SelectGroup(container, event, group)
   container:ReleaseChildren()
   if group == "tab1" then
      self:DrawGroup1(container)
   elseif group == "tab2" then
      self:DrawGroup2(container)
   end
end
